@@ -4,22 +4,17 @@
  *
  * Copyright (C) 2004 NVIDIA Corporation.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of Version 2 of the GNU General Public
- * License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See Version 2
- * of the GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the:
- *
- *           Free Software Foundation, Inc.
- *           59 Temple Place - Suite 330
- *           Boston, MA 02111-1307, USA
- *
+ * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
 #include <gtk/gtk.h>
@@ -96,6 +91,19 @@ static const char *__info_help =
 "This section describes basic informations about the "
 "DVI connection to the digital flat panel.";
 
+static const char *__info_chip_location_help = 
+"Report whether the flat panel is driven by the on-chip controller "
+"(internal), or a separate controller chip elsewhere on the graphics "
+"board (external).";
+                      
+static const char *__info_link_help = 
+"Report whether the specified display device is driven by a single "
+"link or dual link DVI connection.";
+    
+static const char *__info_signal_help = 
+"Report whether the flat panel is driven by an LVDS, TMDS, or "
+"DisplayPort signal.";
+
 static const char * __native_res_help =
 "The Native Resolution is the width and height in pixels that the flat "
 "panel uses to display the image.  All other resolutions must be scaled "
@@ -132,6 +140,22 @@ static const char * __force_gpu_scaling_help =
 "panel's EDID; the flat panel will then scale the image to "
 "its native resolution.";
 
+static const char *__gpu_scaling_help = 
+"Reports whether the GPU and/or DFP are actively scaling the current "
+"resolution.";
+
+static const char *__gpu_scaling_stretched_help = 
+"The image will be expanded to fit the entire flat panel.";
+
+static const char *__gpu_scaling_centered_help = 
+"The image will only occupy the number of pixels "
+"needed and be centered on the flat panel.  Setting this "
+"will disable image sharpening for the display device.";
+
+static const char *__gpu_scaling_ar_scaled_help = 
+"The image will be scaled (retaining the original "
+"aspect ratio) to expand and fit as much of the entire "
+"flat panel as possible.";
 
 GType ctk_display_device_dfp_get_type(void)
 {
@@ -197,13 +221,15 @@ GtkWidget* ctk_display_device_dfp_new(NvCtrlAttributeHandle *handle,
     GtkWidget *banner;
     GtkWidget *frame;
     GtkWidget *hbox, *vbox, *tmpbox;
-    GtkWidget *eventbox;
 
     GtkWidget *button;
     GtkWidget *radio0;
     GtkWidget *radio1;
     GtkWidget *radio2;
     GtkWidget *alignment;
+    GtkWidget *notebook;
+    GtkWidget *nbox;
+    GtkWidget *align;
     
     GtkWidget *table;
     ReturnStatus ret1, ret2;
@@ -255,7 +281,7 @@ GtkWidget* ctk_display_device_dfp_new(NvCtrlAttributeHandle *handle,
      */
 
     ctk_display_device_dfp->reset_button =
-        gtk_button_new_with_label("Reset Hardware Defaults");
+        gtk_button_new_with_label("Reset DFP Hardware Defaults");
 
     alignment = gtk_alignment_new(1, 1, 0, 0);
     gtk_container_add(GTK_CONTAINER(alignment),
@@ -267,13 +293,26 @@ GtkWidget* ctk_display_device_dfp_new(NvCtrlAttributeHandle *handle,
                      (gpointer) ctk_display_device_dfp);
     
     ctk_config_set_tooltip(ctk_config, ctk_display_device_dfp->reset_button,
-                           "The Reset Hardware Defaults button restores "
-                           "the DFP settings to their default values.");
+                           ctk_help_create_reset_hardware_defaults_text("DFP", name));
+
+    /* Create tabbed notebook for widget */
+
+    notebook = gtk_notebook_new();
+    gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP);
+    align = gtk_alignment_new(0, 0, 1, 1);
+    gtk_container_add(GTK_CONTAINER(align), notebook);
+    gtk_box_pack_start(GTK_BOX(object), align, FALSE, FALSE, 0);
+
+    /* Create first tab for dfp info */
+    
+    nbox = gtk_vbox_new(FALSE, FRAME_PADDING);
+    gtk_container_set_border_width(GTK_CONTAINER(nbox), FRAME_PADDING);
+    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), nbox, gtk_label_new("Information"));
 
     /* create the hbox to store dfp info, scaling */
 
     hbox = gtk_hbox_new(FALSE, FRAME_PADDING);
-    gtk_box_pack_start(GTK_BOX(object), hbox, FALSE, FALSE, FRAME_PADDING);
+    gtk_box_pack_start(GTK_BOX(nbox), hbox, FALSE, FALSE, FRAME_PADDING);
 
     /* DFP info */
 
@@ -287,6 +326,7 @@ GtkWidget* ctk_display_device_dfp_new(NvCtrlAttributeHandle *handle,
      */
     
     tmpbox = gtk_vbox_new(FALSE, 5);
+    gtk_container_set_border_width(GTK_CONTAINER(tmpbox), FRAME_PADDING);
     gtk_container_add(GTK_CONTAINER(frame), tmpbox);
     
     /* Make the txt widgets that will get updated */
@@ -311,17 +351,17 @@ GtkWidget* ctk_display_device_dfp_new(NvCtrlAttributeHandle *handle,
             {
                 gtk_label_new("Chip location:"),
                 ctk_display_device_dfp->txt_chip_location,
-                NULL
+                __info_chip_location_help
             },
             {
                 gtk_label_new("Connection link:"),
                 ctk_display_device_dfp->txt_link,
-                NULL
+                __info_link_help
             },
             {
                 gtk_label_new("Signal:"),
                 ctk_display_device_dfp->txt_signal,
-                NULL
+                __info_signal_help
             },
             {
                 gtk_label_new("Native Resolution:"),
@@ -389,14 +429,15 @@ GtkWidget* ctk_display_device_dfp_new(NvCtrlAttributeHandle *handle,
                                FALSE, TRUE, 5);
 
             /* Include tooltips */
-            if (!lines[i].tooltip) {
-                gtk_box_pack_start(GTK_BOX(tmpbox), tmphbox, FALSE, FALSE, 0);
-            } else {
-                eventbox = gtk_event_box_new();
-                gtk_container_add(GTK_CONTAINER(eventbox), tmphbox);
-                ctk_config_set_tooltip(ctk_config, eventbox, lines[i].tooltip);
-                gtk_box_pack_start(GTK_BOX(tmpbox), eventbox, FALSE, FALSE, 0);
+            if (lines[i].tooltip) {
+                ctk_config_set_tooltip(ctk_config, 
+                                       lines[i].label, 
+                                       lines[i].tooltip);
+                ctk_config_set_tooltip(ctk_config, 
+                                       lines[i].txt, 
+                                       lines[i].tooltip);
             }
+            gtk_box_pack_start(GTK_BOX(tmpbox), tmphbox, FALSE, FALSE, 0);
         }
     }
 
@@ -404,13 +445,9 @@ GtkWidget* ctk_display_device_dfp_new(NvCtrlAttributeHandle *handle,
     /* Flat Panel Scaling */
     
     frame = gtk_frame_new("Flat Panel Scaling");
-    eventbox = gtk_event_box_new();
-    gtk_container_add(GTK_CONTAINER(eventbox), frame);
-    gtk_box_pack_start(GTK_BOX(hbox), eventbox, FALSE, FALSE, 0);
-    ctk_display_device_dfp->scaling_frame = eventbox;
+    gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE, FALSE, 0);
+    ctk_display_device_dfp->scaling_frame = frame;
     
-    ctk_config_set_tooltip(ctk_config, eventbox, __scaling_help);
-
     vbox = gtk_vbox_new(FALSE, FRAME_PADDING);
     gtk_container_set_border_width(GTK_CONTAINER(vbox), FRAME_PADDING);
     gtk_container_add(GTK_CONTAINER(frame), vbox);
@@ -428,9 +465,10 @@ GtkWidget* ctk_display_device_dfp_new(NvCtrlAttributeHandle *handle,
     gtk_container_set_border_width(GTK_CONTAINER(table), 5);
     
     ctk_display_device_dfp->txt_scaling = 
-        add_table_row(table, 0,
-                      0, 0.5, "Scaling:",
-                      0, 0.5,  "");
+        add_table_row_with_help_text(table, ctk_config, __gpu_scaling_help,
+                                     0, 0, 
+                                     0, 0.5, "Scaling:",
+                                     0, 0.5,  "");
 
     gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
 
@@ -447,29 +485,48 @@ GtkWidget* ctk_display_device_dfp_new(NvCtrlAttributeHandle *handle,
     radio0 = make_scaling_radio_button
         (ctk_display_device_dfp, vbox, NULL, "Stretched",
          NV_CTRL_GPU_SCALING_METHOD_STRETCHED);
+    ctk_config_set_tooltip(ctk_config, radio0, __gpu_scaling_stretched_help);
     
     radio1 = make_scaling_radio_button
         (ctk_display_device_dfp, vbox, radio0, "Centered",
          NV_CTRL_GPU_SCALING_METHOD_CENTERED);
+    ctk_config_set_tooltip(ctk_config, radio1, __gpu_scaling_centered_help);
     
     radio2 = make_scaling_radio_button
         (ctk_display_device_dfp, vbox, radio1, "Aspect Ratio Scaled",
          NV_CTRL_GPU_SCALING_METHOD_ASPECT_SCALED);
+    ctk_config_set_tooltip(ctk_config, radio2, __gpu_scaling_ar_scaled_help);
     
     g_signal_connect(G_OBJECT(ctk_event),
                      CTK_EVENT_NAME(NV_CTRL_GPU_SCALING),
                      G_CALLBACK(dfp_update_received),
                      (gpointer) ctk_display_device_dfp);
 
+    /* pack the EDID button */
+
+    hbox = gtk_hbox_new(FALSE, 0);
+    align = gtk_alignment_new(0, 1, 1, 1);
+    gtk_container_add(GTK_CONTAINER(align), hbox);
+    gtk_box_pack_end(GTK_BOX(nbox), align, FALSE, FALSE, 0);
+    ctk_display_device_dfp->edid_box = hbox;
+
+    /* 
+     * Create layout for second tab for dfp controls but don't
+     * add the tab until we make sure its required 
+     */
+
+    nbox = gtk_vbox_new(FALSE, FRAME_PADDING);
+    gtk_container_set_border_width(GTK_CONTAINER(nbox), FRAME_PADDING);
+
     /* pack the color controls */
 
     ctk_display_device_dfp->color_controls =
         ctk_color_controls_new(handle, ctk_config, ctk_event,
-                                   ctk_display_device_dfp->reset_button,
-                                   display_device_mask, name);
+                               ctk_display_device_dfp->reset_button,
+                               display_device_mask, name);
 
     if (ctk_display_device_dfp->color_controls) {
-        gtk_box_pack_start(GTK_BOX(object),
+        gtk_box_pack_start(GTK_BOX(nbox),
                            ctk_display_device_dfp->color_controls,
                            FALSE, FALSE, 0);
     }
@@ -482,7 +539,7 @@ GtkWidget* ctk_display_device_dfp_new(NvCtrlAttributeHandle *handle,
                                    display_device_mask, name);
 
     if (ctk_display_device_dfp->dithering_controls) {
-        gtk_box_pack_start(GTK_BOX(object),
+        gtk_box_pack_start(GTK_BOX(nbox),
                            ctk_display_device_dfp->dithering_controls,
                            FALSE, FALSE, 0);
     }
@@ -494,17 +551,19 @@ GtkWidget* ctk_display_device_dfp_new(NvCtrlAttributeHandle *handle,
                               ctk_display_device_dfp->reset_button,
                               display_device_mask, name);
     if (ctk_display_device_dfp->image_sliders) {
-        gtk_box_pack_start(GTK_BOX(object),
+        gtk_box_pack_start(GTK_BOX(nbox),
                            ctk_display_device_dfp->image_sliders,
                            FALSE, FALSE, 0);
     }
 
-    /* pack the EDID button */
+    /* If no controls are created, don't add a controls tab */
+
+    if (ctk_display_device_dfp->color_controls || 
+        ctk_display_device_dfp->dithering_controls || 
+        ctk_display_device_dfp->image_sliders) {
+            gtk_notebook_append_page(GTK_NOTEBOOK(notebook),nbox,gtk_label_new("Controls"));
+    }
 
-    hbox = gtk_hbox_new(FALSE, 0);
-    gtk_box_pack_start(GTK_BOX(object), hbox, FALSE, FALSE, 0);
-    ctk_display_device_dfp->edid_box = hbox;
-    
     /* show the page */
 
     gtk_widget_show_all(GTK_WIDGET(object));
@@ -883,6 +942,7 @@ GtkTextBuffer *ctk_display_device_dfp_create_help(GtkTextTagTable *table,
 {
     GtkTextIter i;
     GtkTextBuffer *b;
+    GtkTooltipsData *td;
     
     b = gtk_text_buffer_new(table);
     
@@ -894,19 +954,13 @@ GtkTextBuffer *ctk_display_device_dfp_create_help(GtkTextTagTable *table,
     ctk_help_para(b, &i, __info_help);
         
     ctk_help_term(b, &i, "Chip Location");
-    ctk_help_para(b, &i, "Report whether the flat panel is driven by "
-                  "the on-chip controller (internal), or a "
-                  "separate controller chip elsewhere on the "
-                  "graphics board (external).");
+    ctk_help_para(b, &i, __info_chip_location_help);
                       
     ctk_help_term(b, &i, "Link");
-    ctk_help_para(b, &i, "Report whether the specified display device "
-                  "is driven by a single link or dual link DVI "
-                  "connection.");
+    ctk_help_para(b, &i, __info_link_help);
     
     ctk_help_term(b, &i, "Signal");
-    ctk_help_para(b, &i, "Report whether the flat panel is driven by "
-                  "an LVDS, TMDS, or DisplayPort signal.");
+    ctk_help_para(b, &i, __info_signal_help);
 
     ctk_help_term(b, &i, "Native Resolution");
     ctk_help_para(b, &i, __native_res_help);
@@ -930,22 +984,20 @@ GtkTextBuffer *ctk_display_device_dfp_create_help(GtkTextTagTable *table,
     ctk_help_para(b, &i, __force_gpu_scaling_help);
     
     ctk_help_term(b, &i, "Scaling");
-    ctk_help_para(b, &i, "Reports whether the GPU and/or DFP are actively "
-                  "scaling the current resolution.");
+    ctk_help_para(b, &i, __gpu_scaling_help);
     
     ctk_help_term(b, &i, "Stretched");
-    ctk_help_para(b, &i, "The image will be expanded to fit the entire "
-                  "flat panel.");
+    ctk_help_para(b, &i, __gpu_scaling_stretched_help);
     
     ctk_help_term(b, &i, "Centered");
-    ctk_help_para(b, &i, "The image will only occupy the number of pixels "
-                  "needed and be centered on the flat panel.  Setting this "
-                  "will disable image sharpening for the display device.");
+    ctk_help_para(b, &i, __gpu_scaling_centered_help);
     
     ctk_help_term(b, &i, "Aspect Ratio Scaled");
-    ctk_help_para(b, &i, "The image will be scaled (retaining the original "
-                  "aspect ratio) to expand and fit as much of the entire "
-                  "flat panel as possible.");
+    ctk_help_para(b, &i, __gpu_scaling_ar_scaled_help);
+
+    if (ctk_display_device_dfp->edid) {
+        add_acquire_edid_help(b, &i);
+    }
 
     if (ctk_display_device_dfp->color_controls) {
         add_color_controls_help
@@ -960,11 +1012,8 @@ GtkTextBuffer *ctk_display_device_dfp_create_help(GtkTextTagTable *table,
     add_image_sliders_help
         (CTK_IMAGE_SLIDERS(ctk_display_device_dfp->image_sliders), b, &i);
     
-    if (ctk_display_device_dfp->edid) {
-        add_acquire_edid_help(b, &i);
-    }
-
-    ctk_help_reset_hardware_defaults(b, &i, ctk_display_device_dfp->name);
+    td = gtk_tooltips_data_get(GTK_WIDGET(ctk_display_device_dfp->reset_button));
+    ctk_help_reset_hardware_defaults (b, &i, td->tip_text);
 
     ctk_help_finish(b);
     
@@ -973,6 +1022,73 @@ GtkTextBuffer *ctk_display_device_dfp_create_help(GtkTextTagTable *table,
 } /* ctk_display_device_dfp_create_help() */
 
 
+static void update_link(CtkDisplayDeviceDfp *ctk_display_device_dfp)
+{
+    ReturnStatus ret;
+    gint val, signal_type = ctk_display_device_dfp->signal_type;
+    const char *link = "Unknown";
+    char tmp[32];
+
+    ret =
+        NvCtrlGetDisplayAttribute(ctk_display_device_dfp->handle,
+                                  ctk_display_device_dfp->display_device_mask,
+                                  NV_CTRL_FLATPANEL_LINK, &val);
+    if (ret == NvCtrlSuccess) {
+        if (signal_type == NV_CTRL_FLATPANEL_SIGNAL_DISPLAYPORT) {
+            int lanes;
+
+            lanes = val + 1;
+
+            ret =
+                NvCtrlGetDisplayAttribute(ctk_display_device_dfp->handle,
+                                          ctk_display_device_dfp->display_device_mask,
+                                          NV_CTRL_DISPLAYPORT_LINK_RATE, &val);
+            if (ret == NvCtrlSuccess && val == NV_CTRL_DISPLAYPORT_LINK_RATE_DISABLED) {
+                link = "Disabled";
+            } else {
+                const char *bw = "unknown bandwidth";
+
+                if (ret == NvCtrlSuccess) {
+                    switch (val) {
+                    case NV_CTRL_DISPLAYPORT_LINK_RATE_1_62GBPS:
+                        bw = "1.62 Gbps";
+                        break;
+                    case NV_CTRL_DISPLAYPORT_LINK_RATE_2_70GBPS:
+                        bw = "2.70 Gbps";
+                        break;
+                    }
+                }
+
+                snprintf(tmp, 32, "%d lane%s @ %s", lanes, lanes == 1 ? "" : "s",
+                         bw);
+                link = tmp;
+            }
+        } else {
+            // LVDS or TMDS
+            switch(val) {
+            case NV_CTRL_FLATPANEL_LINK_SINGLE:
+                link = "Single";
+                break;
+            case NV_CTRL_FLATPANEL_LINK_DUAL:
+                link = "Dual";
+                break;
+            }
+        }
+    }
+
+    gtk_label_set_text
+        (GTK_LABEL(ctk_display_device_dfp->txt_link), link);
+}
+
+
+static void callback_link_changed(GtkObject *object, gpointer arg1,
+                                  gpointer user_data)
+{
+    CtkDisplayDeviceDfp *ctk_display_device_dfp =
+        CTK_DISPLAY_DEVICE_DFP(user_data);
+
+    update_link(ctk_display_device_dfp);
+}
 
 /*
  * dfp_info_setup() -
@@ -982,10 +1098,10 @@ GtkTextBuffer *ctk_display_device_dfp_create_help(GtkTextTagTable *table,
 static void dfp_info_setup(CtkDisplayDeviceDfp *ctk_display_device_dfp)
 {
     ReturnStatus ret;
-    gint val, signal_type, gpu_scaling, dfp_scaling;
+    gint val, gpu_scaling, dfp_scaling;
     char *chip_location, *link, *signal;
     char *scaling;
-    char tmp[32];
+    CtkEvent *ctk_event = ctk_display_device_dfp->ctk_event;
 
     chip_location = link = signal = "Unknown";
     scaling = "Unknown";
@@ -1030,59 +1146,21 @@ static void dfp_info_setup(CtkDisplayDeviceDfp *ctk_display_device_dfp)
     }
     gtk_label_set_text
         (GTK_LABEL(ctk_display_device_dfp->txt_signal), signal);
-    signal_type = val;
+    ctk_display_device_dfp->signal_type = val;
 
     /* Link */
 
-    ret =
-        NvCtrlGetDisplayAttribute(ctk_display_device_dfp->handle,
-                                  ctk_display_device_dfp->display_device_mask,
-                                  NV_CTRL_FLATPANEL_LINK, &val);
-    if (ret == NvCtrlSuccess) {
-        if (signal_type == NV_CTRL_FLATPANEL_SIGNAL_DISPLAYPORT) {
-            int lanes;
+    g_signal_connect(G_OBJECT(ctk_event),
+                     CTK_EVENT_NAME(NV_CTRL_FLATPANEL_LINK),
+                     G_CALLBACK(callback_link_changed),
+                     (gpointer) ctk_display_device_dfp);
 
-            lanes = val + 1;
-
-            ret =
-                NvCtrlGetDisplayAttribute(ctk_display_device_dfp->handle,
-                                          ctk_display_device_dfp->display_device_mask,
-                                          NV_CTRL_DISPLAYPORT_LINK_RATE, &val);
-            if (ret == NvCtrlSuccess && val == NV_CTRL_DISPLAYPORT_LINK_RATE_DISABLED) {
-                link = "Disabled";
-            } else {
-                char *bw = "unknown bandwidth";
-
-                if (ret == NvCtrlSuccess) {
-                    switch (val) {
-                    case NV_CTRL_DISPLAYPORT_LINK_RATE_1_62GBPS:
-                        bw = "1.62 Gbps";
-                        break;
-                    case NV_CTRL_DISPLAYPORT_LINK_RATE_2_70GBPS:
-                        bw = "2.70 Gbps";
-                        break;
-                    }
-                }
-
-                snprintf(tmp, 32, "%d lane%s @ %s", lanes, lanes == 1 ? "" : "s",
-                         bw);
-                link = tmp;
-            }
-        } else {
-            // LVDS or TMDS
-            switch(val) {
-            case NV_CTRL_FLATPANEL_LINK_SINGLE:
-                link = "Single";
-                break;
-            case NV_CTRL_FLATPANEL_LINK_DUAL:
-                link = "Dual";
-                break;
-            }
-        }
-    }
-    gtk_label_set_text
-        (GTK_LABEL(ctk_display_device_dfp->txt_link), link);
+    g_signal_connect(G_OBJECT(ctk_event),
+                     CTK_EVENT_NAME(NV_CTRL_DISPLAYPORT_LINK_RATE),
+                     G_CALLBACK(callback_link_changed),
+                     (gpointer) ctk_display_device_dfp);
 
+    update_link(ctk_display_device_dfp);
 
     /* Native Resolution */
 
